package bpc.samples;

import java.util.Date;
import java.text.FieldPosition;
import java.text.SimpleDateFormat;

public class AuditTrail {
	
	private static final SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	private static final FieldPosition fp = new FieldPosition(SimpleDateFormat.FULL);
	private static final String NEWLINE = "\n";
	private static final String INDENT   = "  ";
	private static final String COLON    = ":";
	
	public static StringBuffer start()
	{
		StringBuffer sb = new StringBuffer(256);
		sb.append("Process started at ");
		sdf.format(new Date(),sb,fp);
		
		return sb;
	}
	
	public static StringBuffer begin(String s)
	{
		StringBuffer sb = null;
		
		if(s != null && s.length() > 0)
		{
			sb = new StringBuffer(s.length() + 512);
			sb.append(s);
			sb.append(NEWLINE);
		}
		else
		{
			sb = new StringBuffer(512);
		}
		//add timestamp
		sdf.format(new Date(),sb,fp);
		sb.append("  Updates:");
		
		return sb;
	}
	
	public static void updateState(StringBuffer auditTrail, int oldState, int newState)
	{
		if(oldState != newState)
		{
			updateString(auditTrail,"State",Integer.toString(oldState),Integer.toString(newState));
		}
	}
	
	public static void updateWaitingTime(StringBuffer auditTrail, int waitingTime)
	{
		if(waitingTime > 0)
		{
			auditTrail.append("\n  Waiting time has been set to ");
			auditTrail.append(waitingTime);
			auditTrail.append(" minutes.");
		}
	}
	
	public static void addError(StringBuffer auditTrail, String error)
	{
		if(error != null && error.length() > 0)
		{
			addString(auditTrail,"Error",error);
		}
	}
	
	public static void updateSupplierNo(StringBuffer auditTrail, String oldSupplierNo, String newSupplierNo)
	{
		updateString(auditTrail,"Supplier number",oldSupplierNo,newSupplierNo);
	}
	
	public static void updateOrderText(StringBuffer auditTrail, String oldOrderText, String newOrderText)
	{
		updateString(auditTrail,"Order text",oldOrderText,newOrderText);
	}

	public static void addComment(StringBuffer auditTrail, String comment)
	{
		if(comment != null && comment.length() > 0)
		{
			addString(auditTrail,"A comment has been added",comment);
		}
	}
	
	public static void updateSizeOfOrder(StringBuffer auditTrail, double oldSize, double newSize)
	{
		if(oldSize != newSize)
		{
			updateString(auditTrail,"Size of order",Double.toString(oldSize),Double.toString(newSize));
		}
	}
	
	public static void updateString(StringBuffer auditTrail, String item, String oldString, String newString)
	{
		if(different(oldString, newString))
		{
			auditTrail.append(NEWLINE);
			auditTrail.append(INDENT);
			auditTrail.append(item);
			auditTrail.append(" has been changed from ");
			auditTrail.append(oldString);
			auditTrail.append(" to ");
			auditTrail.append(newString);
			auditTrail.append(".");			
		}
	}
	

	public static void addString(StringBuffer auditTrail, String item, String s)
	{
		if(s != null && s.length() > 0)
		{
			auditTrail.append(NEWLINE);
			auditTrail.append(INDENT);
			if(item != null && item.length() > 0)
			{
				auditTrail.append(item);
				auditTrail.append(COLON);
				auditTrail.append(NEWLINE);
				auditTrail.append(INDENT);
				auditTrail.append(INDENT);
			}
			auditTrail.append(s);
		}
	}
	
	private static boolean different(String left, String right)
	{
		if(left == null)
		{
			return  (right != null  && right.length() > 0);
		}
		else
		{
			return !( left.equals(right) );
		}
	}
}
